import Image

def scale(val, maxVal, floatMin, floatMax):
    """ scales a value in a range 0 to maxVal into the range between
        floatMin and floatMax
    """
    return floatMin + (val / float(maxVal)) * (floatMax - floatMin)

def msetColor( width, coordinateList = [-2.1,0.8,-1.3,1.3], numIterations = 25 ):
    """ saves an image of the Mandelbrot set.
        inputs:
        width (integer) of image
        coordinateList (optional list of floats) : [minX, maxX, minY, maxY]
        numIterations (optional int)
    """
        
    # Calculate various useful values
    [min_x, max_x, min_y, max_y] = coordinateList    
    height = int( width*(max_y - min_y)/(max_x - min_x) )
    
    # Initialize the image
    image = Image.new('RGB', (width, height), (255, 255, 255) )
    pix = image.load()
    
    # Make arrays of the points that we'll be iterating over
    yscale = float(max_y - min_y) / float(height)
    ylocs = [float(y) * yscale + min_y for y in range(height)]
    xscale = float(max_x - min_x) / float(width)
    xlocs = [float(x) * xscale + min_x for x in range(width)]

    # Iterate over points, saving colored points in the mandelbrot set.    
    for row, yloc in enumerate(ylocs):
        for column, xloc in enumerate(xlocs):
            colored_value = int(10.2 * inMSet( xloc, yloc, numIterations ))
            if  colored_value != -10:
                pix[column, row] = (0, 0, 0)
                

    file_name = "pics/MB_Py"+str(numIterations)+"_"+str(width)+".png"
    image.save(file_name)

def inMSet(real, imag, n):
    
    currReal = real
    currImag = imag
    
    for counter in range(n):
        [currReal, currImag] = [currReal ** 2 - currImag ** 2 + real,
                                2 * currReal * currImag + imag]
        if currReal ** 2 + currImag ** 2 > 4:
            return counter

    return -1
        
    
