cimport decl

def msetColor(int width, coordinateList = [-2.1,0.8,-1.3,1.3],
              int numIterations = 25 ):
    """ Saves an image of the Mandelbrot set.
        Inputs:
        width (integer) of image
        coordinateList (optional list of floats) : [minX, maxX, minY, maxY]
        numIterations (optional int)
    """
    cdef int row, column, height
    cdef float min_x, max_x, min_y, max_y
    cdef float real, imag
    cdef int colored_value

    # Calculate various useful constants
    [min_x, max_x, min_y, max_y] = coordinateList    
    height = int( width*(max_y - min_y)/(max_x - min_x) )
    
    # Initialize the image
    cdef decl.Image* image
    image = new decl.Image()
    image.Create(width, height, decl.Color(255, 255, 255))
        
    # Make arrays of the points that we'll be iterating over
    cdef float yscale = float(max_y - min_y) / float(height)
    ylocs = [y * yscale + min_y for y in range(height)]
    cdef float xscale = float(max_x - min_x) / float(width)
    xlocs = [x * xscale + min_x for x in range(width)]

    # Iterate over points, saving colored points in the mandelbrot set.    
    for row, yloc in enumerate(ylocs):
        for column, xloc in enumerate(xlocs):
            colored_value = inMSet( xloc, yloc, numIterations)
            if  colored_value >= 0:
                image.SetPixel(column, row, decl.Color())
                

    filename = "pics/MB_SF"+str(numIterations)+"_"+str(width)+".png"
    
    save_to_file(image, filename)

cdef inline save_to_file(decl.Image* image, char* filename):
    image.SaveToFile(filename)
    
cdef inline int inMSet(double real, double imag, int n):
    
    cdef int counter
    cdef double currReal = real, currImag = imag
    
    for counter in range(n):
        [currReal, currImag] = [currReal ** 2 - currImag ** 2 + real,
                                2 * currReal * currImag + imag]
        if currReal ** 2 + currImag ** 2 > 4:
            return counter

    return -1
        
    
